@@ -64,6 +64,8 @@ int conn_sock_drain(struct connection *conn);
 int conn_send_socks4_proxy_request(struct connection *conn);
 int conn_recv_socks4_proxy_response(struct connection *conn);
 
+__decl_hathreads(extern HA_SPINLOCK_T toremove_lock[MAX_THREADS]);
+
 /* returns true is the transport layer is ready */
 static inline int conn_xprt_ready(const struct connection *conn)
 {
@@ -352,8 +354,7 @@ static inline void conn_sock_shutw(struct connection *c, int clean)
 	/* don't perform a clean shutdown if we're going to reset or
 	 * if the shutr was already received.
 	 */
-	if (conn_ctrl_ready(c) && !(c->flags & CO_FL_SOCK_RD_SH) && clean &&
-	    !fdtab[c->handle.fd].linger_risk)
+	if (conn_ctrl_ready(c) && !(c->flags & CO_FL_SOCK_RD_SH) && clean)
 		shutdown(c->handle.fd, SHUT_WR);
 }
 
@@ -596,7 +597,9 @@ static inline void conn_free(struct connection *conn)
 	}
 
 	conn_force_unsubscribe(conn);
+	HA_SPIN_LOCK(OTHER_LOCK, &toremove_lock[tid]);
 	LIST_DEL_LOCKED(&conn->list);
+	HA_SPIN_UNLOCK(OTHER_LOCK, &toremove_lock[tid]);
 	pool_free(pool_head_connection, conn);
 }
 
@@ -1092,6 +1095,9 @@ static inline int conn_install_mux_be(struct connection *conn, void *ctx, struct
 	return conn_install_mux(conn, mux_ops, ctx, prx, sess);
 }
 
+/* Change the mux for the connection.
+ * The caller should make sure he's not subscribed to the underlying XPRT.
+ */
 static inline int conn_upgrade_mux_fe(struct connection *conn, void *ctx, struct buffer *buf,
 				      struct ist mux_proto, int mode)
 {
@@ -1119,7 +1125,6 @@ static inline int conn_upgrade_mux_fe(struct connection *conn, void *ctx, struct
 	old_mux_ctx = conn->ctx;
 	conn->mux = new_mux;
 	conn->ctx = ctx;
-	conn_force_unsubscribe(conn);
 	if (new_mux->init(conn, bind_conf->frontend, conn->owner, buf) == -1) {
 		/* The mux upgrade failed, so restore the old mux */
 		conn->ctx = old_mux_ctx;
