@@ -34,36 +34,45 @@
 
 #include <proto/backend.h>
 
-extern struct pool_head *pool2_pendconn;
+extern struct pool_head *pool_head_pendconn;
 
-int init_pendconn();
-struct stream *pendconn_get_next_strm(struct server *srv, struct proxy *px);
 struct pendconn *pendconn_add(struct stream *strm);
-void pendconn_free(struct pendconn *p);
+int pendconn_dequeue(struct stream *strm);
 void process_srv_queue(struct server *s);
 unsigned int srv_dynamic_maxconn(const struct server *s);
 int pendconn_redistribute(struct server *s);
 int pendconn_grab_from_px(struct server *s);
+void pendconn_unlink(struct pendconn *p);
 
-
-/* Returns the first pending connection for server <s>, which may be NULL if
- * nothing is pending.
+/* Removes the pendconn from the server/proxy queue. It supports being called
+ * with NULL for pendconn and with a pendconn not in the list. It is the
+ * function to be used by default when unsure. Do not call it with server
+ * or proxy locks held however.
  */
-static inline struct pendconn *pendconn_from_srv(const struct server *s) {
-	if (!s->nbpend)
-		return NULL;
-
-	return LIST_ELEM(s->pendconns.n, struct pendconn *, list);
+static inline void pendconn_cond_unlink(struct pendconn *p)
+{
+	if (p && p->node.node.leaf_p)
+		pendconn_unlink(p);
 }
 
-/* Returns the first pending connection for proxy <px>, which may be NULL if
- * nothing is pending.
+/* Releases the pendconn associated to stream <s> if it has any, and decreases
+ * the pending count if needed. The connection might have been queued to a
+ * specific server as well as to the proxy. The stream also gets marked
+ * unqueued.
+ *
+ * This function must be called by the stream itself, so in the context of
+ * process_stream, without any lock held among the pendconn, the server's queue
+ * nor the proxy's queue.
  */
-static inline struct pendconn *pendconn_from_px(const struct proxy *px) {
-	if (!px->nbpend)
-		return NULL;
+static inline void pendconn_free(struct stream *s)
+{
+	struct pendconn *p = s->pend_pos;
 
-	return LIST_ELEM(px->pendconns.n, struct pendconn *, list);
+	if (p) {
+		pendconn_cond_unlink(p);
+		s->pend_pos = NULL;
+		pool_free(pool_head_pendconn, p);
+	}
 }
 
 /* Returns 0 if all slots are full on a server, or 1 if there are slots available. */
@@ -76,10 +85,29 @@ static inline int server_has_room(const struct server *s) {
  * for and if/else usage.
  */
 static inline int may_dequeue_tasks(const struct server *s, const struct proxy *p) {
-	return (s && (s->nbpend || (p->nbpend && srv_is_usable(s))) &&
+	return (s && (s->nbpend || (p->nbpend && srv_currently_usable(s))) &&
 		(!s->maxconn || s->cur_sess < srv_dynamic_maxconn(s)));
 }
 
+static inline int queue_limit_class(int class)
+{
+	if (class < -0x7ff)
+		return -0x7ff;
+	if (class > 0x7ff)
+		return 0x7ff;
+	return class;
+}
+
+static inline int queue_limit_offset(int offset)
+{
+	if (offset < -0x7ffff)
+		return -0x7ffff;
+	if (offset > 0x7ffff)
+		return 0x7ffff;
+	return offset;
+}
+
+
 #endif /* _PROTO_QUEUE_H */
 
 /*
