@@ -13,21 +13,25 @@
 
 #include <signal.h>
 #include <common/standard.h>
+#include <common/hathreads.h>
+
 #include <types/signal.h>
 #include <types/task.h>
 
 extern int signal_queue_len;
 extern struct signal_descriptor signal_state[];
-extern struct pool_head *pool2_sig_handlers;
+
+__decl_hathreads(extern HA_SPINLOCK_T signals_lock);
 
 void signal_handler(int sig);
 void __signal_process_queue();
-int signal_init();
 void deinit_signals();
 struct sig_handler *signal_register_fct(int sig, void (*fct)(struct sig_handler *), int arg);
 struct sig_handler *signal_register_task(int sig, struct task *task, int reason);
 void signal_unregister_handler(struct sig_handler *handler);
 void signal_unregister_target(int sig, void *target);
+void signal_unregister(int sig);
+void haproxy_unblock_signals();
 
 static inline void signal_process_queue()
 {
