@@ -27,10 +27,11 @@ struct cli_kw {
 	const char *str_kw[5];   /* keywords ended by NULL, limited to 5
 				 separated keywords combination */
 	const char *usage;   /* usage message */
-	int (*parse)(char **args, struct appctx *appctx, void *private);
+	int (*parse)(char **args, char *payload, struct appctx *appctx, void *private);
 	int (*io_handler)(struct appctx *appctx);
 	void (*io_release)(struct appctx *appctx);
 	void *private;
+	int level; /* this is the level needed to show the keyword usage and to use it */
 };
 
 struct cli_kw_list {
