@@ -24,6 +24,8 @@ struct shared_block {
 	unsigned int len;          /* data length for the row */
 	unsigned int block_count;  /* number of blocks */
 	unsigned int refcount;
+	struct shared_block *last_reserved;
+	struct shared_block *last_append;
 	unsigned char data[0];
 };
 
@@ -38,6 +40,7 @@ struct shared_context {
 	struct list avail;  /* list for active and free blocks */
 	struct list hot;     /* list for locked blocks */
 	unsigned int nbav;  /* number of available blocks */
+	unsigned int max_obj_size;   /* maximum object size (in bytes). */
 	void (*free_block)(struct shared_block *first, struct shared_block *block);
 	short int block_size;
 	unsigned char data[0];
