@@ -38,11 +38,29 @@ enum si_state {
 	SI_ST_ASS,               /* server just assigned to this interface */
 	SI_ST_CON,               /* initiated connection request (resource exists) */
 	SI_ST_CER,               /* [transient] previous connection attempt failed (resource released) */
+	SI_ST_RDY,               /* [transient] ready proven after I/O success during SI_ST_CON */
 	SI_ST_EST,               /* connection established (resource exists) */
 	SI_ST_DIS,               /* [transient] disconnected from other side, but cleanup not done yet */
 	SI_ST_CLO,               /* stream intf closed, might not existing anymore. Buffers shut. */
 } __attribute__((packed));
 
+/* state bits for use with lists of states */
+enum si_state_bit {
+	SI_SB_NONE = 0,
+	SI_SB_INI = 1U << SI_ST_INI,
+	SI_SB_REQ = 1U << SI_ST_REQ,
+	SI_SB_QUE = 1U << SI_ST_QUE,
+	SI_SB_TAR = 1U << SI_ST_TAR,
+	SI_SB_ASS = 1U << SI_ST_ASS,
+	SI_SB_CON = 1U << SI_ST_CON,
+	SI_SB_CER = 1U << SI_ST_CER,
+	SI_SB_RDY = 1U << SI_ST_RDY,
+	SI_SB_EST = 1U << SI_ST_EST,
+	SI_SB_DIS = 1U << SI_ST_DIS,
+	SI_SB_CLO = 1U << SI_ST_CLO,
+	SI_SB_ALL = SI_SB_INI|SI_SB_REQ|SI_SB_QUE|SI_SB_TAR|SI_SB_ASS|SI_SB_CON|SI_SB_CER|SI_SB_RDY|SI_SB_EST|SI_SB_DIS|SI_SB_CLO,
+};
+
 /* error types reported on the streams interface for more accurate reporting */
 enum {
 	SI_ET_NONE       = 0x0000,  /* no error yet, leave it to zero */
@@ -64,7 +82,7 @@ enum {
 	SI_FL_NONE       = 0x00000000,  /* nothing */
 	SI_FL_EXP        = 0x00000001,  /* timeout has expired */
 	SI_FL_ERR        = 0x00000002,  /* a non-recoverable error has occurred */
-	/* unused:         0x00000004 */
+	SI_FL_KILL_CONN  = 0x00000004,  /* next shutw must kill the whole conn, not just the stream */
 	SI_FL_WAIT_DATA  = 0x00000008,  /* stream-int waits for more outgoing data to send */
 	SI_FL_ISBACK     = 0x00000010,  /* 0 for front-side SI, 1 for back-side */
 	SI_FL_DONT_WAKE  = 0x00000020,  /* resync in progress, don't wake up */
@@ -72,7 +90,7 @@ enum {
 	SI_FL_NOLINGER   = 0x00000080,  /* may close without lingering. One-shot. */
 	SI_FL_NOHALF     = 0x00000100,  /* no half close, close both sides at once */
 	SI_FL_SRC_ADDR   = 0x00001000,  /* get the source ip/port with getsockname */
-	/* unused:         0x00002000 */
+	/* unused: 0x00000200 */
 	SI_FL_WANT_GET   = 0x00004000,  /* a stream-int would like to get some data from the buffer */
 	SI_FL_CLEAN_ABRT = 0x00008000,  /* SI_FL_ERR is used to report aborts, and not SHUTR */
 
@@ -83,6 +101,8 @@ enum {
 	SI_FL_RXBLK_CONN = 0x00100000,  /* other side is not connected */
 	SI_FL_RXBLK_ANY  = 0x001F0000,  /* any of the RXBLK flags above */
 	SI_FL_RX_WAIT_EP = 0x00200000,  /* stream-int waits for more data from the end point */
+	SI_FL_L7_RETRY   = 0x01000000,  /* The stream interface may attempt L7 retries */
+	SI_FL_D_L7_RETRY = 0x02000000,  /* Disable L7 retries on this stream interface, even if configured to do it */
 };
 
 /* A stream interface has 3 parts :
@@ -111,6 +131,7 @@ struct stream_interface {
 	int conn_retries;	/* number of connect retries left */
 	unsigned int hcto;      /* half-closed timeout (0 = unset) */
 	struct wait_event wait_event; /* We're in a wait list */
+	struct buffer l7_buffer; /* To store the data, in case we have to retry */
 };
 
 /* operations available on a stream-interface */
