@@ -56,7 +56,7 @@ int check_trk_action(struct act_rule *rule, struct proxy *px, char **err)
 		return 0;
 	}
 	else {
-		if (target->proxies_list != px) {
+		if (!in_proxies_list(target->proxies_list, px)) {
 			px->next_stkt_ref = target->proxies_list;
 			target->proxies_list = px;
 		}
