@@ -72,7 +72,7 @@ static void _update_fd(int fd)
 		if (!(fdtab[fd].thread_mask & tid_bit) || !(en & FD_EV_POLLED_RW)) {
 			/* fd removed from poll list */
 			opcode = EPOLL_CTL_DEL;
-			HA_ATOMIC_AND(&polled_mask[fd], ~tid_bit);
+			_HA_ATOMIC_AND(&polled_mask[fd], ~tid_bit);
 		}
 		else {
 			/* fd status changed */
@@ -82,7 +82,7 @@ static void _update_fd(int fd)
 	else if ((fdtab[fd].thread_mask & tid_bit) && (en & FD_EV_POLLED_RW)) {
 		/* new fd in the poll list */
 		opcode = EPOLL_CTL_ADD;
-		HA_ATOMIC_OR(&polled_mask[fd], tid_bit);
+		_HA_ATOMIC_OR(&polled_mask[fd], tid_bit);
 	}
 	else {
 		return;
@@ -103,7 +103,7 @@ static void _update_fd(int fd)
 /*
  * Linux epoll() poller
  */
-REGPRM2 static void _do_poll(struct poller *p, int exp)
+REGPRM3 static void _do_poll(struct poller *p, int exp, int wake)
 {
 	int status;
 	int fd;
@@ -116,7 +116,7 @@ REGPRM2 static void _do_poll(struct poller *p, int exp)
 	for (updt_idx = 0; updt_idx < fd_nbupdt; updt_idx++) {
 		fd = fd_updt[updt_idx];
 
-		HA_ATOMIC_AND(&fdtab[fd].update_mask, ~tid_bit);
+		_HA_ATOMIC_AND(&fdtab[fd].update_mask, ~tid_bit);
 		if (!fdtab[fd].owner) {
 			activity[tid].poll_drop++;
 			continue;
@@ -147,7 +147,7 @@ REGPRM2 static void _do_poll(struct poller *p, int exp)
 	thread_harmless_now();
 
 	/* now let's wait for polled events */
-	wait_time = compute_poll_timeout(exp);
+	wait_time = wake ? 0 : compute_poll_timeout(exp);
 	tv_entering_poll();
 	activity_count_runtime();
 	do {
@@ -160,7 +160,7 @@ REGPRM2 static void _do_poll(struct poller *p, int exp)
 			break;
 		if (timeout || !wait_time)
 			break;
-		if (signal_queue_len)
+		if (signal_queue_len || wake)
 			break;
 		if (tick_isset(exp) && tick_is_expired(exp, now_ms))
 			break;
@@ -186,7 +186,7 @@ REGPRM2 static void _do_poll(struct poller *p, int exp)
 			/* FD has been migrated */
 			activity[tid].poll_skip++;
 			epoll_ctl(epoll_fd[tid], EPOLL_CTL_DEL, fd, &ev);
-			HA_ATOMIC_AND(&polled_mask[fd], ~tid_bit);
+			_HA_ATOMIC_AND(&polled_mask[fd], ~tid_bit);
 			continue;
 		}
 
@@ -208,7 +208,7 @@ REGPRM2 static void _do_poll(struct poller *p, int exp)
 
 		/* always remap RDHUP to HUP as they're used similarly */
 		if (e & EPOLLRDHUP) {
-			HA_ATOMIC_OR(&cur_poller.flags, HAP_POLL_F_RDHUP);
+			_HA_ATOMIC_OR(&cur_poller.flags, HAP_POLL_F_RDHUP);
 			n |= FD_POLL_HUP;
 		}
 		fd_update_events(fd, n);
